with Lower_Layer_UDP;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Strings.Unbounded;
with Ada.Exceptions;

package body Paq is


   -- Tipos de mensajes de control:
   -- 1: Mensaje con el nombre para entrar en el chat
   -- 2: Mensaje de respuesta con lista al intentar acceder al chat
   -- 3: Peticin de mensaje privado
   -- 4: Mensaje para salir del chat
   -- 5: Asentimiento de privado
   -- 6: Rechazo de privado
   -- 7: Terminar privado
   -- 8: Peticin de informacin adicional
   -- 9: Recibo informacin adicional


   -- Comandos:
   -- .s    -> salir
   -- .ex   -> escuchar en el canal x
   -- .dx   -> dejar de escuchar en el canal x
   -- .cx   -> cambiarte para escribir al canal x
   -- .l    -> lista de usuarios del chat
   -- .im   -> Informa sobre el canal al que mando mensajes en ese momento determinado
   -- .ie   -> Informa sobre los canales en los que se reciben mensajes en ese momento determinado
   -- .p x  -> Peticin de privado con el usuario x
   -- .pf   -> Terminar conversacin privada
   -- .m x  -> Informacin adicional sobre el usuario x
   -- .a    -> ayuda: muestra los comandos disponibles

   Max_Lis: constant := 50;
   Num_Can: constant := 5;
   type
     T_Lista is array (1..Max_Lis) of ASU.Unbounded_String;

   Vacio: Boolean;  -- Para saber si el chat est vaco, es decir, no hay nadie
   Dentro: Boolean; -- Para saber si estoy dentro del chat o no
   Lista: T_Lista;  -- Lista para guardar los nombres
   Ultimo_Lista: Natural;  -- Guarda el nmero de nombres en la lista
   Control: LLU.End_Point_Type;  -- Direccin para mensajes de control
   Mensajes: LLU.End_Point_Type; -- Direccin para  mandar los mensajes
   Salas:  array ( 1..Num_Can) of LLU.End_Point_Type;  -- Direcciones de las cinco salas
   Nombre: String (1..25);  -- Se guarda el nombre del usuario
   Long_Nombre: Natural;  -- Longitud del nombre de usuario
   Inf_Adicional: String (1..30); -- Se guarda el correo electrnico del usuario
   Long_Inf: Natural;  -- Longitud de la informacin adicional
   Tipo: Natural;  -- Tipo de los mensajes
   Dir_Unicast: LLU.End_Point_Type;  -- Direccin unicast de cada usuario

   Canal_Manda: Natural; -- Guarda el nmero del canal en el que se escriben los mensajes
   Canal_Ant: Natural;  -- Se guarda el canal anterior al que estaba antes de comenzar un privado, para poder volver luego a l
   Canal_Recibe: array (1..Max_Lis) of Boolean;  -- Indica los canales de los que se reciben mensajes
   Aux: Natural;
   Privado: Boolean; -- para saber si mantiene una conversacin privada


   -- ************************************************************************************************************************
   -- -------------------------    IMPRIME LA LISTA DE USUARIOS EN LA PANTALLA     -------------------------------------------
   -- ************************************************************************************************************************

   procedure Imprimir_Lista is
   begin
      Put_Line ("La lista es:");
      for I in 1..Ultimo_Lista loop
         Put_Line (ASU.To_String(Lista(i)));
      end loop;
   end Imprimir_Lista;


   -- ************************************************************************************************************************
   -- ------------------------------    PROCEDIMIENTO DE ENTRADA AL CHAT      ------------------------------------------------
   -- ************************************************************************************************************************

   procedure Entrada_Chat is
      Nom: ASU.Unbounded_String;  -- NOMBRE DEL USUARIO
      Buffer: aliased LLU.Buffer_Type (1024);
   begin

      -- CONSTRUYO LA DIRECCIN DE CONTROL Y ME ATO A ELLA
      Control := LLU.Build ("232.28.24.23",23023);
      LLU.Bind (Control, Recibir_Control'Access);

      -- ME ATO A UNA DIRECCIN UNICAST
      LLU.Bind_Any (Dir_Unicast, Recibir_Mensajes'Access);

      -- SE REPITE EL BUCLE HASTA QUE EL USUARIO CONSIGUE ENTRAR EN EL CHAT
      loop
         Privado:=False;
         Dentro:=False;
         Vacio:=True;

         -- INICIALIZO LA LISTA DE NOMBRES
         for I in 1..  Max_Lis loop
            Lista(I):= ASU.To_Unbounded_String ("");
         end loop;
         Ultimo_Lista:= 0;

         Put ("INTRODUCE TU NOMBRE: ");
         Get_Line (Nombre,Long_Nombre);
         Put ("Direccin de correo electrnico: ");
         Get_Line (Inf_Adicional, Long_Inf);

         -- MANDO LA PETICIN DE ENTRADA CON MI NOMBRE, Y MI DIRECCIN UNICAST
         LLU.Reset (Buffer);
         Nom:= ASU.To_Unbounded_String (Nombre(1..Long_Nombre));
         Natural'Output (Buffer'Access, 1); -- Mensaje tipo 1: de entrada al chat
         ASU.Unbounded_String'Output (Buffer'Access, Nom);
         LLU.End_Point_Type'Output (Buffer'Access, Dir_Unicast);
         LLU.Send (Control, Buffer'Access);

         -- ESPERO 2 SEGUNDOS
         delay (2.0);

         -- SI DESPUS DE 2 SEGUNDOS NADIE ME HA RESPONDIDO, SUPONGO QUE EL CHAT EST VACO
         if Vacio then
            Dentro:= True;
            Ultimo_Lista:=1;
            Lista(1):= Nom;
            Put_Line ("");
            Put_Line ("Bienvenido al chat ... ");
            Put_Line ("(Pulsa .a para ayuda)");
            Put_Line ("");
         end if;
         exit when Dentro;
      end loop;

      -- CONSTRUYO LAS DIRECCIONES DE LOS 5 CANALES
      Salas(1):= LLU.Build ("232.28.24.24", 23024);
      Salas(2):= LLU.Build ("232.28.24.25", 23025);
      Salas(3):= LLU.Build ("232.28.24.26", 23026);
      Salas(4):= LLU.Build ("232.28.24.27", 23027);
      Salas(5):= LLU.Build ("232.28.24.28", 23028);

      -- ME ATO A LA PRIMERA SALA, POR DEFECTO, PARA RECIBIR MENSAJES
      LLU.Bind (Salas(1), Recibir_Mensajes'Access);
      Canal_Recibe(1):= True;

      -- ESCRIBO POR DEFECTO TAMBIN EN LA PRIMERA SALA
      Canal_Manda:=1;
      Mensajes:= Salas (1);
      for I in 2..Num_Can loop
         Canal_Recibe (I):=False;
      end loop;
   end Entrada_Chat;


   -- ************************************************************************************************************************
   -- ---------------------   PROCEDMIENTO PARA LEER DE TECLADO Y EJECUTAR COMANDOS O MANDAR MENSAJES   ----------------------
   -- ************************************************************************************************************************

   procedure Lectura_Envio (Salir: in out Boolean) is
      S: String (1..255); -- CADENA QUE LEO DE TECLADO
      Long: Natural;  -- LONGITUD DE LA CADENA QUE LEO
      Mensaje: ASU.Unbounded_String;
      Buffer: aliased LLU.Buffer_Type (1024);
      Nomb: ASU.Unbounded_String;  -- NOMBRE DEL PROPIO USUARIO
      N: ASU.Unbounded_String;

   begin

      LLU.Reset (Buffer);
      Ada.Text_IO.Put (">");
      Ada.Text_IO.Get_Line (S, Long);
      Mensaje:= ASU.To_Unbounded_String (S(1..Long));
      Nomb:= ASU.To_Unbounded_String (Nombre(1..Long_Nombre));

      -- SI LA CADENA EMPIEZA POR . SIGNIFICA QUE ES UN COMANDO
      if S(1)='.' then
         case S(2) is

            -- PARA ABANDONAR EL CHAT
            when 's' => begin
               if not Privado then
                  Salir:=True;
               else
                  Put_Line ("ERROR: Primero debes abandonar el privado");
               end if;
            end;

            -- PARA VISUALIZAR LA LISTA DE USUARIOS
            when 'l' => Imprimir_Lista;

            -- PARA ESCUCHAR EN ALGN CANAL
            when 'e' => begin

               -- AUX CONTIENE EL CANAL AL QUE SE QUIERE ESCUCHAR
               Aux:= Character'Pos (S(3))- Character'Pos('0');

               if (Aux<Num_Can+1) and (Aux>0) then -- SI EL NMERO DE CANAL ES CORRECTO
                  if (Canal_Recibe(Aux)=True) then  -- YA ESCUCHABA EN ESE CANAL
                     Put_Line ("ERROR: Ya estabas escuchando en el canal" & Integer'Image(Aux));
                  else
                     Put_Line ("Ahora escuchars del canal " & Integer'Image (Aux));
                     Canal_Recibe (Aux):= True;
                     LLU.Bind (Salas(Aux), Recibir_Mensajes'Access);
                  end if;
               else
                  Put_Line ("ERROR: El canal " & Integer'Image(Aux) & " no existe");
               end if;
            end;

            -- PARA DEJAR DE ESCUCHAR DE ALGN CANAL
            when 'd' => begin

               -- AUX CONTIENE EL CANAL AL QUE SE QUIERE DEJAR DE ESCUCHAR
               Aux:= Character'Pos (S(3)) - Character'Pos('0');
               if (Aux<Num_Can+1) and (Aux>0) then  -- SI EL NMERO DE CANAL ES CORRECTO
                  if (Canal_Recibe(Aux)) then  -- SI ESCUCHABAS ANTES DEL CANAL
                     Put_Line ("Ya no escuchars en el canal " & Integer'Image (Aux));
                     Canal_Recibe (Aux):= False;
                     LLU.UnBind (Salas(Aux));
                  else
                     Put_Line ("ERROR: No estabas escuchando en el canal " & Integer'Image (Aux));
                  end if;
               else
                  Put_Line ("ERROR: El canal " & Integer'Image(Aux) & " no existe");
               end if;
            end;

            -- PARA CAMBIAR EL CANAL AL QUE SE MANDAN MENSAJES
            when 'c' => begin
               if not Privado then
                  Aux:= Character'Pos (S(3)) - Character'Pos ('0');
                  if (Aux<Num_Can+1) and (Aux>0) then
                     if (Canal_Manda=Aux) then
                        Put_Line ("ERROR: Ya estabas en el canal " & Integer'Image (Aux));
                     else
                        Canal_Manda:= Aux;
                        Mensajes:= Salas (Aux);
                        Put_Line ("Ahora ests en el canal " & Integer'Image (Aux));
                     end if;
                  else
                     Put_Line ("ERROR: El canal " & Integer'Image(Aux) & " no existe");
                  end if;
               else
                  Put_Line ("ERROR: Primero debes abandonar el privado");
               end if;
            end;

            -- PARA VER INFORMACIN SOBRE LOS CANALES A LOS QUE SE MANDAN Y RECIBEN LOS MENSAJES
            when 'i' => begin
               case S(3) is

                  -- INFORMACIN DEL CANAL AL QUE SE MANDA
                  when 'm' => begin
                     Put_Line ("");
                     Put ("   Actualmente mandas al canal: ");
                     if Canal_Manda=0 then
                        Put_Line ("PRIVADO");
                     else
                        Put_Line ( Integer'Image (Canal_Manda));
                     end if;
                     Put_Line ("");
                  end;

                  -- INFORMACIN DE LOS CANALES DE LOS QUE SE ESCUCHA
                  when 'e' => begin
                     Put_Line ("");
                     Put_Line ("   Actualmente escuchas de los canales: ");
                     Put ("          ");
                     for I in 1..Num_Can loop
                        if Canal_Recibe (I) then Put (" " & Integer'Image (I) & " ");
                        end if;
                     end loop;
                     if Privado then
                        Put ("PRIVADO");
                     end if;
                     Put_Line ("");
                  end;
                  when others => Put_Line ("ERROR: Comando no vlido ");
               end case;
            end;

            -- PETICIN DE PRIVADO Y FINALIZACIN DE PRIVADO
            when 'p' => begin
               -- FINALIZACIN DE PRIVADO
               if (Long=3) and (S(3)='f') then
                  if Privado then

                     -- MANDA UN MENSAJE AL OTRO PARA TERMINAR LA CONVERSACIN PRIVADA
                     LLU.Reset(Buffer);
                     Natural'Output (Buffer'Access, 7);
                     ASU.Unbounded_String'Output (Buffer'Access, Nomb);
                     LLU.End_Point_Type'Output (Buffer'Access, Mensajes);
                     LLU.Send (Control, Buffer'Access);

                     Put_Line ("Ha terminado la conversacin privada");
                     Canal_Manda:= Canal_Ant;
                     Mensajes:= Salas(Canal_Manda);
                     Privado:=False;
                  else
                     Put_Line ("ERROR: Actualmente no mantienes conversaciones privadas");
                  end if;
               else
                  -- PETICIN DE PRIVADO
                  if not Privado then
                     N:= ASU.To_Unbounded_String(S(4..Long));

                     -- MANDA UN MENSAJE AL OTRO PARA PEDIRLE UN PRIVADO
                     LLU.Reset (Buffer);
                     Natural'Output (Buffer'Access, 3);
                     ASU.Unbounded_String'Output (Buffer'Access, N);
                     ASU.Unbounded_String'Output (Buffer'Access, Nomb);
                     LLU.End_Point_Type'Output (Buffer'Access,Dir_Unicast);
                     LLU.Send (Control, Buffer'Access);
                  else
                     Put_Line ("ERROR: Solo pueden mantener un privado");
                  end if;
               end if;

            end;

            -- PARA VER LA DIRECCIN DE CORREO ELECTRNICO DE ALGN USUARIO
            when 'm' => begin
               N:= ASU.To_Unbounded_String (S(4..Long));

               -- MANDO LA PETICIN AL USUARIO PARA QUE ME ENVE SU DIRECCIN DE CORREO
               LLU.Reset (Buffer);
               Natural'Output (Buffer'Access, 8);
               ASU.Unbounded_String'Output (Buffer'Access, N);
               ASU.Unbounded_String' Output (Buffer'Access, Nomb);
               LLU.Send (Control, Buffer'Access);
            end;

            -- VISUALIZA LA AYUDA EN PANTALLA
            when 'a' => begin
               Put_Line ("Los comandos disponibles son: ");
               Put_Line ("----------------------------- ");
               Put_Line (".s:  Abandonar el chat ");
               Put_Line (".l:  Lista de usuarios del chat ");
               Put_Line (".m x: Correo electrnico de x");
               Put_Line (".ex: Escuchar en el canal x ");
               Put_Line (".dx: Dejar de escuchar en el canal x ");
               Put_Line (".cx: Cambiarse al canal x ");
               Put_Line (".im: Canal al que mando mensajes");
               Put_Line (".ie: Canales de los que escucho");
               Put_Line (".p x: Comenzar conversacin privada con x");
               Put_Line (".pf: terminar conversacin privada");
               Put_Line (".a:  Ayuda ");

            end;
            when others => Put_Line ("ERROR: Comando no vlido ");
         end case;
      else
         -- SI ES UN MENSAJE NORMAL, LO ENVO A LA DIRECCIN DE LOS MENSAJES, AL CANAL AL QUE ESCRIBA EN ESE MOMENTO
         Salir:=False;
         Natural'Output (Buffer'Access, Canal_Manda);
         ASU.Unbounded_String'Output (Buffer'Access, Nomb);
         ASU.Unbounded_String'Output (Buffer'Access, Mensaje);
         LLU.Send (Mensajes, Buffer'Access);
      end if;
   end Lectura_Envio;



   -- ************************************************************************************************************************
   -- ---------------------------------   PROCEDMIENTO PARA SALIR DEL CHAT    ------------------------------------------------
   -- ************************************************************************************************************************
   procedure Salir_Chat is
      Nomb: ASU.Unbounded_String;
      Buffer: aliased LLU.Buffer_Type (1024);
   begin
      Nomb:= ASU.To_Unbounded_String (Nombre(1..Long_Nombre));

      -- MANDO UN MENSAJE PARA INFORMAR A LOS DEMS DE QUE ABANDONO EL CHAT
      LLU.Reset (Buffer);
      Natural'Output (Buffer'Access, 4); -- Mensaje tipo 4
      ASU.Unbounded_String'Output (Buffer'Access,Nomb);
      LLU.Send (Control, Buffer'Access);

      -- ME DESATO DE TODAS LAS DIRECCIONES
      LLU.Unbind (Dir_Unicast);
      LLU.Unbind (Control);
      for I in 1..Num_Can loop
         if Canal_Recibe(I) then LLU.UnBind (Salas(I)); end if;
      end loop;

   end Salir_Chat;



   -- ************************************************************************************************************************
   -- ----------------   MANEJADOR QUE RECIBE LOS MENSAJES DE CONTROL QUE LLEGAN A LA DIRECCIN DE CONTROL   -----------------
   -- ************************************************************************************************************************

   procedure Recibir_Control (Dir: in LLU.End_Point_Type; Buffer_A: access LLU.Buffer_Type) is
      Mensaje: ASU.Unbounded_String;
      Nom: ASU.Unbounded_String;
      Aux: ASU.Unbounded_String;
      Encontrado: Boolean;
      Buffer: aliased LLU.Buffer_Type (1024);
      Dir_Uni:LLU.End_Point_Type;
   begin
      Tipo:= Natural'Input (Buffer_A); -- Extraigo el tipo del mensaje
      Nom:= ASU.Unbounded_String'Input (Buffer_A);

      if Dentro then -- SI EL PROCESO YA EST DENTRO DEL CHAT
         case Tipo is

            -- PETICIN DE ENTRADA AL CHAT
            when 1 => begin

               -- SI SOY EL LTIMO, MANDO LA LISTA AL QUE INTENTA ENTRAR
               if ASU.To_String (Lista(Ultimo_Lista))= Nombre(1..Long_Nombre) then
                  Dir_Uni:= LLU.End_Point_Type'Input (Buffer_A);
                  LLU.Reset (Buffer);
                  Natural'Output (Buffer'Access, 2); -- Mensaje con la lista
                  ASU.Unbounded_String'Output (Buffer'Access, Nom);
                  LLU.End_Point_Type'Output (Buffer'Access, Dir_Uni);
                  Natural'Output (Buffer'Access, Ultimo_Lista);
                  T_Lista'Output (Buffer'Access, Lista);
                  LLU.Send (Dir, Buffer'Access);
              end if;

              -- COMPRUEBO SI EXISTE EL NOMBRE EN LA LISTA
              Encontrado:= False;
              for I in 1..Ultimo_Lista loop
                 if ASU.To_String(Lista(I))= ASU.To_String (Nom) then
                    Encontrado:=True;
                 end if;
              end loop;

              -- SI NO EXISTE EL NOMBRE, LO METO EN LA LISTA
              if not Encontrado then
                 Ultimo_Lista:=Ultimo_Lista+1;
                 Lista(Ultimo_Lista):= Nom;
                 Put_Line (ASU.To_String (Nom) & " HA ENTRADO AL CHAT");
                 Put (">");
              end if;
            end;

            -- PETICIN DE PRIVADO: Compruebo si es para mi el privado. Si lo es recibo la peticin de privado,
            -- y si no estoy ya en un privado le mando un asentimiento del privado, pero si ya estoy, le mando
            -- paquete para indicarle que no acepto el privado
            when 3 => begin
               if ASU.To_String(Nom) =Nombre(1..Long_Nombre) then

                  if not Privado then  -- ACEPTA EL PRIVADO
                     Put_Line ("");
                     Put (" Comienza la conversacin privada con ");
                     Put_Line (ASU.To_String (ASU.Unbounded_String'Input (Buffer_A)) & " .......");
                     Put_Line ("");
                     Put (">");
                     Dir_Uni:= LLU.End_Point_Type'Input (Buffer_A);
                     Privado:= True;
                     Canal_Ant:= Canal_Manda;
                     Canal_Manda:=0;
                     Mensajes:= Dir_Uni;
                     LLU.Reset (Buffer);
                     Natural'Output (Buffer'Access,5);
                     ASU.Unbounded_String'Output (Buffer'Access, Nom);
                     LLU.End_Point_Type'Output (Buffer'Access, Dir_Uni);
                     LLU.End_Point_Type'Output (Buffer'Access, Dir_Unicast);
                     LLU.Send (Control, Buffer'Access);

                  else  -- RECHAZA EL PRIVADO
                     Aux:= ASU.Unbounded_String'Input (Buffer_A);
                     Dir_Uni:= LLU.End_Point_Type'Input (Buffer_A);
                     LLU.Reset (Buffer);
                     Natural'Output (Buffer'Access,6);
                     ASU.Unbounded_String'Output (Buffer'Access, ASU.To_Unbounded_String (Nombre(1..Long_Nombre)));
                     LLU.End_Point_Type'Output (Buffer'Access, Dir_Uni);
                     LLU.Send (Control, Buffer'Access);
                  end if;
               end if;
            end;

            -- MENSAJE QUE MANDA ALGN USUARIO QUE ABANDONA EL CHAT
            when 4=>begin
               -- BORRO EL NOMBRE DE LA LISTA, Y LA ACTUALIZO
               for I in 1..Ultimo_Lista loop
                  if ASU.To_String(Lista(I))=ASU.To_String(Nom) then
                     Lista(I):=ASU.To_Unbounded_String("");
                     if Ultimo_Lista>1 then
                        Lista(I):= Lista(Ultimo_Lista);
                        Lista(Ultimo_Lista):=ASU.To_Unbounded_String("");
                        Ultimo_Lista:= Ultimo_Lista-1;
                     end if;
                  end if;
               end loop;
               Put_Line (ASU.To_String (Nom) & " SALE DEL CHAT");
               Put (">");
            end;

            -- RECIBE EL ASENTIMIENTO DE LA CONVERSACIN PRIVADA
            when 5 => begin
               Dir_Uni:= LLU.End_Point_Type'Input (Buffer_A);
               if LLU."=" (Dir_Uni, Dir_Unicast) then
                  Privado:= True;
                  Canal_Ant:= Canal_Manda;
                  Canal_Manda:=0;
                  Mensajes:= LLU.End_Point_Type'Input (Buffer_A);
                  Put_Line ("");
                  Put_Line (" Comienza la conversacin privada con " & ASU.To_String (Nom) & " .......");
                  Put_Line ("");
                  Put (">");
               end if;
            end;

            -- RECIBE UN RECHAZO DE PRIVADO
            when 6 => begin
               Dir_Uni:= LLU.End_Point_Type'Input (Buffer_A);
               if LLU."="(Dir_Uni, Dir_Unicast) then
                  Put_Line (ASU.To_String (Nom) & " rechaza el privado ");
                  Put (">");
               end if;
            end;

            -- PARA TERMINAR LA CONVERSACIN PRIVADA: Vuelvo al canal al que estaba antes de la conversacin privada
            when 7 => begin
               Dir_Uni:= LLU.End_Point_Type'Input (Buffer_A);
               if LLU."=" (Dir_Uni, Dir_Unicast) then
                  Canal_Manda:= Canal_Ant;
                  Mensajes:= Salas(Canal_Manda);
                  Put_Line ("");
                  Put_Line (ASU.To_String (Nom) & " ha abandonado la conversacin privada");
                  Put (">");
                  Privado:=False;
               end if;
            end;

            -- RECIBO PETICIN DE INFORMACIN ADICIONAL Y SE LA MANDO
            when 8 => begin
               if ASU.To_String (Nom)= Nombre (1..Long_Nombre) then
                  Nom:= ASU.Unbounded_String'Input (Buffer_A);
                  LLU.Reset (Buffer);
                  Natural'Output (Buffer'Access, 9);
                  ASU.Unbounded_String'Output (Buffer'Access, Nom);
                  ASU.Unbounded_String'Output (Buffer'Access, ASU.To_Unbounded_String (Inf_Adicional(1..Long_Inf)));
                  LLU.Send (Control, Buffer'Access);
               end if;
            end;

            -- RECIBO LA INFORMACIN ADICIONAL QUE HABA SOLICITADO
            when 9 => begin
               if ASU.To_String (Nom)= Nombre (1..Long_Nombre) then
                  Mensaje:= ASU.Unbounded_String'Input (Buffer_A);
                  Put_Line ("");
                  Put_Line ("Correo: " & ASU.To_String (Mensaje));
                  Put_Line ("");
                  Put (">");
               end if;
            end;
            when others => null;
         end case;

      else  -- SI TODAVA NO HE ENTRADO AL CHAT

         case Tipo is

            -- MENSAJE DONDE SE RECIBE LA LISTA
            when 2=>begin
               Dir_Uni:= LLU.End_Point_Type'Input (Buffer_A);
               if LLU."="(Dir_Uni,Dir_Unicast) then  -- COMPRUEBO SI LA LISTA ME LA MANDAN A MI
                 Vacio:=False;
                 Ultimo_Lista:= Natural'Input (Buffer_A);
                 Lista:= T_Lista'Input (Buffer_A);

                 -- MIRO EN LA LISTA SI EST MI NOMBRE
                 Encontrado:=False;
                 for I in 1..Ultimo_Lista loop
                    if ASU.To_String(Lista(I))=ASU.To_String(Nom) then
                       Encontrado:=True;
                    end if;
                 end loop;

                 -- EST MI NOMBRE NO ENTRO
                 if Encontrado then
                    Dentro:=False;
                    Put_Line ("EL NOMBRE YA EXISTE");
                 else  -- SI MI NOMBRE NO EST EN LA LISTA ENTRO AL CHAT
                    Put_Line ("");
                    Put_Line ("Bienvenido al chat ...");
                    Put_Line ("(Pulsa .a para ayuda)");
                    Put_Line ("");
                    Dentro:=True;
                    Ultimo_Lista:=Ultimo_Lista+1;
                    Lista (Ultimo_Lista):= Nom;
                 end if;
              end if;
            end;
            when others => null;
         end case;
      end if;
      LLU.Reset(Buffer_A.all);
   end Recibir_Control;



   -- ************************************************************************************************************************
   -- --------------------------------   MANEJADOR QUE RECIBE LOS MENSAJES DEL CHAT    ---------------------------------------
   -- ************************************************************************************************************************

   procedure Recibir_Mensajes (Dir: in LLU.End_Point_Type; Buffer_A: access LLU.Buffer_Type) is
      Can: Natural;
      Nom: ASU.Unbounded_String;
      Mensaje: ASU.Unbounded_String;
   begin
      -- EXTRAIGO EL NMERO DE CANAL DE DONDE VIENE EL MENSAJE
      can:= Natural'Input (Buffer_A);

      -- EXTRAIGO EL NOMBRE DE USUARIO QUE MANDA EL MENSAJE
      Nom:= ASU.Unbounded_String'Input (Buffer_A);

      -- SI EL MENSAJE NO ES MO
      if Nombre(1..Long_Nombre)/= ASU.To_String(Nom) then
         Mensaje:= ASU.Unbounded_String'Input (Buffer_A);
         if Can=0 then
            Put ("PRIVADO");
         else
            Put (Integer'Image (Can));
         end if;
         Put (" < " & ASU.To_String (Nom) & ": ");
         Put_Line (ASU.To_String (Mensaje));
         Put (">");
      end if;
      LLU.Reset (Buffer_A.all);
   end;

end Paq;
